[ORG 0x7C00]                ; offsets calculated from 0:0x7C00
[BITS 16]                  

start:
    
; - goal - 
; - - setup IDT and ISRs
; - - get timer working
; - - get keyboard working

    ; ------------------------------------------------------------ ;
    ; -- this code sets up the stack and zeros the GP registers -- ;
    ; ------------------------------------------------------------ ;
 .bootok:
    xor ax, ax
    mov ds, ax
    mov es, ax
    mov ss, ax
    mov sp, 0x7B00
    
    call term_clear

    ; --------------------------------------------- ;
    ; -- print the banner and request a password -- ;
    ; --------------------------------------------- ;
    mov si, str_corpbanner
    call term_print
    mov si, str_passreq
    call term_print
    
    call term_get_password

lognums:    
    call term_clear
    mov si, str_corpbanner
    call term_print
    mov si, str_lognum
    call term_print
    
    call term_get_char
    call term_print_char
    
    mov si, str_bye
    call term_print
    
 .tloop:
    call term_get_char
    call term_print_char
    jmp .tloop
     
    hlt
  
   
; ----------------------------------------------------------------- ;
; -- term_wait delays operations for 0x16000 microseconds (90ms) -- ;
; ----------------------------------------------------------------- ;
term_wait:
    pusha
    mov cx, 0x0001
    mov dx, 0x6000
    mov ah, 0x86
    int 0x15
    popa
    ret

; ----------------------------------------------------------------------- ;
; -- term_clear clears the visible screen and resets the cursor to 0,0 -- ;
; ----------------------------------------------------------------------- ;
term_clear:
    pusha
    mov ax, 0x0600
    mov bx, 0x0200
    xor cx, cx
    mov dx, 0x184F
    int 0x10
    
    mov ah, 0x02
    xor bh, bh
    mov dx, 0x0100
    int 0x10
    popa
    ret

; --------------------------------------------------------- ;
; -- term_print prints the ASCII string located at ES:SI -- ;
; --------------------------------------------------------- ;
term_print:
    pusha
 .strloop:
    lodsb                   ; grab a byte from SI
    cmp al, 0               ; have we reached the end of the string?
    je .done                ; if we have, exit

    call term_wait    
    
    mov ah, 0x0E            ; tell INT 0x10 to print to the screen
    int 0x10                ; print the char in AL
    jmp .strloop            ; and loop...
 .done:
    popa
    ret                     ; return

; ------------------------------------------------------------------------------- ;
; -- term_get_char waits for a keypress and returns its ASCII equivelant in AL -- ;
; ------------------------------------------------------------------------------- ;
term_get_char:
    mov ah, 0x10            ; function 0x10 (GET ENHANCED KEYSTROKE)
    int 0x16                ; INT 0x16 
    ret                 
    
term_print_char:
    pusha
    mov ah, 0x0E            ; function 0x0E (TELETYPE OUTPUT)
    mov bx, 0x0002          ; bh = 0 (page num), bl = 2 (forground color)
    int 0x10                ; INT 0x10
    popa
    ret

term_get_string:
    mov bx, 0x7B00
 .start:
    call term_get_char  
    cmp al, 0x08            ; is backspace?
    je .bkspc
    cmp al, 0x0D
    je .enter
    
    cmp bl, 0x08
    jae .start
    
    mov byte [bx], al
    inc bl
    mov byte [bx], 0
    
    cmp dl, 0xFE
    je .noshadow
    mov al, '*'
 .noshadow:
    call term_print_char
    jmp .start
    
 .bkspc:
    cmp bl, 0x00
    je .start
    dec bx
    mov byte [bx], 0
    push ax                 ; push/pop is better than mov al, 0x08... =)
    call term_print_char    ; print the backspace code
    mov al, 0x00            ; set al to a space char
    call term_print_char    ; print the ' ' code
    pop ax                  ; set al to the backspace code
    call term_print_char    ; print the backspace code
    jmp .start              ; and loop...
    
 .enter:
    mov di, 0x7B00
    ret  
        
      
term_get_password:
    mov ax, 0x0535
    push ax
 .loop:
    mov ah, 0x02
    xor bh, bh
    mov dx, 0x0400
    int 0x10
    mov cx, 160
    
 .clear_lines:
    mov al, ' '
    call term_print_char
    loop .clear_lines
    
    mov ah, 0x02
    xor bh, bh
    mov dx, 0x0400
    int 0x10
    
    pop ax
    dec ah
    dec al
    
    cmp ah, 0
    je .lock_terminal
    
    push ax
    call term_print_char
    mov si, str_attempts
    call term_print
    
 .put_attempts:
    mov al, 254
    call term_print_char
    call term_wait
    mov al, ' '
    call term_print_char
    dec ah
    cmp ah, 0
    ja .put_attempts
    
    mov si, str_newline
    call term_print
    
    mov al, '>'
    call term_print_char
    
    call term_get_string
    mov si, str_password
    call str_compare
    cmp al, 1
    jne .loop
    call term_clear
    jmp term_authenticated
    
 .lock_terminal:
    call term_clear
    mov si, str_termlocked
    call term_print
    hlt
  
 .exit:
    ret

term_authenticated:
    mov si, str_corpbanner
    call term_print
    mov si, str_logs
    call term_print
    mov al, '>'
    call term_print_char
    call term_get_char
    
    cmp al, '0'
    je .read_logs
    cmp al, '1'
    je .write_logs
    hlt

 .read_logs:
    jmp lognums
 .write_logs:
    jmp lognums

; ------------------------------------------------------------------------------ ;
; -- str_compare compares two strings (in SI and DI); 1 for true, 0 for false -- ;
; ------------------------------------------------------------------------------ ;
str_compare:
    pusha
 .cmploop:
    mov al, byte [si]
    cmp al, byte [di]
    jne .notequ
    cmp al, 0
    je .equ
    inc si
    inc di
    jmp .cmploop
 .notequ:
    popa
    mov al, 0
    ret 
 .equ:
    popa
    mov al, 1
    ret
        
str_corpbanner  db 'DINS TERMLINK PROTOCOL', 0x0D, 0x0A, 0
str_passreq     db 'ENTER PASSWORD NOW', 0x0D, 0x0A, 0
str_newline     db 0x0D, 0x0A, 0
str_attempts    db ' ATTEMPT(S) LEFT: ', 0
str_password    db 'hackit', 0
str_logs    db '0) VIEW LOGS', 0x0D, 0x0A, '1) WRITE LOGS', 0x0D, 0x0A, 0
str_termlocked  db 'TERMINAL LOCKED', 0
str_lognum      db 'LOG #', 0
str_bye         db 0x0D, 0x0A, 'BYE!', 0
       
TIMES 510-($-$$) DB 0
BIOS_SIG DW 0xAA55




